@@ -134,12 +134,16 @@ class Format(SimpleEnum):
     OBJECTID = 'objectid'  # Default format for ObjectId
     CHECKBOX = 'checkbox'  # Default format for bool
     SWITCH = 'switch'
+    HIDDEN = 'hidden'  # Hidden input
     # Below values are used for inner model/dict or list of model/dict
     LIST = 'list'  # Default format for List
     TAB = 'tab'
     TABLE = 'table'
+    MODAL = 'modal'
     CARD = 'card'
     CAROUSEL = 'carousel'
+    CHART = 'chart'  # Object{title, names, values}
+    STATISTIC = 'statistic'  # Simple value or List[value] or List[{name, value}]
     COLLAPSE = 'collapse'
     CASCADER = 'cascader'  # Cascade selection, i.e, List[str]
     LATLNG = 'latlng'  # LatLng chooser, i.e, List[float]
@@ -257,17 +261,15 @@ class ModelField:
         """
         self.name = name
         self.type = type_
+        # default is none if undefined
         if default is Undefined:
             self.default = None
-            self.required = True
-        elif default is None:
-            self.default = None
-            self.required = False
         else:
             self.default = default
+        # required is true if undefined
+        if required is Undefined:
             self.required = True
-        # Overwrite calcuated required if specified
-        if required is not Undefined:
+        else:
             self.required = required
         self.format = format_
         self.title = title
@@ -325,43 +327,52 @@ class ModelMeta(ABCMeta):
         """ Create and return a new object. """
         fields: Dict[str, ModelField] = {}
         relations: Dict[str, relation] = {}
-
         # TODO: Support callable validators
-
         slots: Set[str] = namespace.get('__slots__', ())
         slots = {slots} if isinstance(slots, str) else set(slots)
+        #
+        for base_ in reversed(bases):
+            if issubclass(base_, BaseModel) and base_ is not BaseModel:
+                fields.update(deepcopy(base_.__fields__))
+        #
+        max_list_level = 0
 
-        for base in reversed(bases):
-            if issubclass(base, BaseModel) and base is not BaseModel:
-                fields.update(deepcopy(base.__fields__))
-
-        def _validate_annotation(field_name, field_type):
-            """ Validate field definition."""
+        def _validate_annotation(field_name, field_type, level):
+            """ Validate field definition. """
+            # Update max level
+            nonlocal max_list_level
+            max_list_level = max(max_list_level, level)
             # Validate if shadows a parent attribute
-            for base in bases:
-                if getattr(base, field_name, None):
+            for base_ in bases:
+                if getattr(base_, field_name, None):
                     raise SchemaError(f'{field_name}: {field_type} shadows a parent attribute')
             #
             origin = get_origin(field_type)
             # Simple, including built-in type, SimpleEnum or sub model
             if origin is None:
-                _validate_simple_annotation(field_name, field_type)
+                _validate_simple_annotation(field_name, field_type, level)
             # Dict
             elif origin is dict:
                 k_type, v_type = get_args(field_type)
                 if k_type is not str:
                     raise SchemaError(f'{field_name}: {field_type} only support str keys')
                 #
-                _validate_simple_annotation(field_name, v_type)
+                _validate_simple_annotation(field_name, v_type, level)
             # List
             elif origin is list:
                 l_type = get_args(field_type)[0]
-                _validate_simple_annotation(field_name, l_type)
+                if l_type in AUTHORIZED_TYPES + [field_type]:  # Skip level of simple types and self-referencing
+                    pass
+                else:
+                    level += 1
+                #
+                _validate_simple_annotation(field_name, l_type, level)
             else:
                 raise SchemaError(f'{field_name}: {field_type} is not supported')
 
-        def _validate_simple_annotation(field_name, field_type):
+        def _validate_simple_annotation(field_name, field_type, level):
             """ Inner method to validate built-in types, SimpleEnums or sub models. """
+            print(f'{field_name} with {field_type}')
             if isinstance(field_type, SimpleEnumMeta):
                 enum_types = set()
                 for member in field_type:
@@ -374,14 +385,16 @@ class ModelMeta(ABCMeta):
                     raise SchemaError(f'{field_name}: {field_type} can not have more than one type')
             elif issubclass(field_type, BaseModel):
                 for a_n, a_t in get_type_hints(field_type).items():
-                    _validate_annotation(a_n, a_t)
+                    _validate_annotation(a_n, a_t, level)
             elif field_type not in AUTHORIZED_TYPES:
                 raise SchemaError(f'{field_name}: {field_type} is not an authorized type')
 
         # Validation
         skips = set()
         if namespace.get('__qualname__') != 'BaseModel':
-            # Annotations
+            #
+            # Annotations(Fields)
+            #
             annotations = namespace.get('__annotations__', {})
             for ann_name, ann_type in annotations.items():
                 # Skip
@@ -389,7 +402,7 @@ class ModelMeta(ABCMeta):
                     skips.add(ann_name)
                     continue
                 # Validate
-                _validate_annotation(ann_name, ann_type)
+                _validate_annotation(ann_name, ann_type, 0)
                 # Create model field
                 field = ModelField(name=ann_name, type_=ann_type)
                 value = namespace.get(ann_name, Undefined)
@@ -403,7 +416,7 @@ class ModelMeta(ABCMeta):
                     field.required = False
                 # Define a ModelField directly
                 elif isinstance(value, ModelField):
-                    # x6
+                    # x7
                     field.default = value.default
                     field.required = value.required
                     field.format = value.format
@@ -432,10 +445,13 @@ class ModelMeta(ABCMeta):
                 fields[ann_name] = field
             #
             # Relations
+            #
             for attr in namespace:
                 if isinstance(namespace.get(attr), relation):
                     relations[attr] = namespace.get(attr)
-
+        # Too many nested list will cause UI generation and business logic too complicated, so raise an error to prevent
+        if max_list_level >= 3:
+            raise SchemaError(f'{name} has too many nested list: {max_list_level}')
         # Create class
         exclude_from_namespace = fields.keys() | skips | relations.keys() | {'__slots__'}
         new_namespace = {
@@ -455,6 +471,7 @@ class BaseModel(metaclass=ModelMeta):
     __doc__ = ''
 
     # TODO: Validate below fields
+    __title__ = None
     __searchables__ = []
     __columns__ = []
     __sortables__ = []
@@ -620,6 +637,9 @@ class BaseModel(metaclass=ModelMeta):
     def __str__(self):
         return f'{self.__class__.__name__}{self.dict()}'
 
+    def __repr__(self):
+        return f'{self.dict()}'
+
     @classmethod
     def get_type(cls, path):
         """ Get type of a path.
@@ -767,8 +787,8 @@ class BaseModel(metaclass=ModelMeta):
           - add type date
           - Add format to array, so that we can gen a component for the whole array
           - Add searchables to root object, so that it can be used to generate search form
-          - Add sortables to root object, so that it can be used to generate order drowpdown
-          - Add columns to root object, so that it can be used to generate columns for table
+          - Add sortables to object, so that it can be used to generate order drowpdown
+          - Add columns to object, so that it can be used to generate columns for table
         """
 
         def _gen_schema(type_: Type):
@@ -779,6 +799,7 @@ class BaseModel(metaclass=ModelMeta):
                     'enum': list(type_),
                     'enum_titles': type_.titles,
                     'format': Format.SELECT,
+                    'py_type': type_.__name__,
                 })
                 return enum
             elif issubclass(type_, BaseModel):
@@ -802,7 +823,8 @@ class BaseModel(metaclass=ModelMeta):
                         })
                     # built-in type, SimpleEnum or sub model
                     else:
-                        field_schema.update(_gen_schema(f_t.type))
+                        inner_type = _gen_schema(f_t.type)
+                        field_schema.update(inner_type)
                     # default
                     if f_t.default:
                         # Skip if default is callable, e.g, datetime.now
@@ -835,7 +857,10 @@ class BaseModel(metaclass=ModelMeta):
                     'type': 'object',
                     'properties': properties,
                     'required': required,
+                    'columns': type_.__columns__ if type_.__columns__ else required,
+                    'sortables': type_.__sortables__ if type_.__sortables__ else [],
                     'py_type': type_.__name__,
+                    'title': type_.__title__ if type_.__title__ else type_.__name__.upper()
                 }
                 # layout
                 layout = []
@@ -847,7 +872,7 @@ class BaseModel(metaclass=ModelMeta):
                             continue
                         layout.append([x.strip() for x in r.split(',')])
                 else:
-                    layout = properties.keys()
+                    layout = [[f] for f in properties.keys()]  # Each field has one row
                 #
                 obj['layout'] = layout
                 #
@@ -871,10 +896,6 @@ class BaseModel(metaclass=ModelMeta):
         searchables = [('%s__%s' % s if isinstance(s, tuple) else s) for s in cls.__searchables__]
         if searchables:
             ret['searchables'] = searchables
-        if cls.__sortables__:
-            ret['sortables'] = cls.__sortables__
-        #
-        ret['columns'] = cls.__columns__ if cls.__columns__ else ret['required']
         #
         # print(json.dumps(ret))
         return ret
